@@ -6,10 +6,10 @@
 //                  Service de Police Judiciaire
 //                  Section Nouvelles Technologies
 // ****************************************************************************
-//  Module:         Multithreaded AFF (AAFF = Avanced AFF)
+//  Module:         Multithreaded AFF (AAFF = Advanced AFF)
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -29,12 +29,14 @@
 #ifndef __AAFF_H__
 #define __AAFF_H__
 
+#include "hash.h"
+
 typedef struct _t_Aaff *t_pAaff;
 
 typedef struct
 {
-   bool Zero;         // Tells AafWrite that all data is 0
-   bool Compressed;   // Tells AafWrite whether the data should be written compressed or uncompresed
+   bool Zero;         // Tells AaffWrite that all data is 0
+   bool Compressed;   // Tells AaffWrite whether the data should be written compressed or uncompressed
    int  DataLenOut;   // If Compressed is true: The size of the compressed data
 } t_AaffPreprocess, *t_pAaffPreprocess;
 
@@ -49,15 +51,16 @@ typedef struct
 //              Functions
 // ------------------------------------
 
+
 // Write access functions
 APIRET AaffOpen         (t_pAaff *ppAaff, const char *pFilename, unsigned long long DeviceSize, unsigned int SectorSize, unsigned int PageSize);
 APIRET AaffPreprocess   (t_pAaffPreprocess *ppPreprocess, unsigned char *pDataIn, unsigned int DataLenIn, unsigned char *pDataOut, unsigned int DataLenOut);
 APIRET AaffWrite        (t_pAaff   pAaff, t_pAaffPreprocess pPreprocess, const unsigned char *pData, unsigned int DataLen);
-APIRET AaffClose        (t_pAaff *ppAaff, unsigned long long BadSectors, const unsigned char *pMD5, const unsigned char *pSHA256, int Duration);
+APIRET AaffClose        (t_pAaff *ppAaff, unsigned long long BadSectors, const unsigned char *pMD5, const unsigned char *pSHA256, uint Duration);
 
 // Read access functions
-APIRET AaffOpen         (t_pAaff *ppAaff, const char *pFilename);
-APIRET AaffReadNextPage (t_pAaff   pAaff, unsigned char *pData, unsigned int *pDataLen);
+APIRET AaffOpen         (t_pAaff *ppAaff, const char *pFilename, unsigned long long DeviceSize);
+APIRET AaffReadNextPage (t_pAaff   pAaff, unsigned char *pData, unsigned int *pDataLen, QString *pImageFilename, t_pHashMD5Digest pMD5Digest, bool *pMD5valid);
 APIRET AaffClose        (t_pAaff *ppAaff);
 
 
@@ -65,6 +68,9 @@ APIRET AaffWriteSegmentStr (t_pAaff pAaff, const char *pName, unsigned int Argum
 
 APIRET AaffCopyBadSectorMarker (unsigned char *pBuffer, unsigned int Len);
 
+// Misc. functions
+unsigned int AaffPreprocessExtraSpace (unsigned int FifoBlockSize);
+
 APIRET AaffInit   (void);
 APIRET AaffDeInit (void);
 
@@ -75,28 +81,24 @@ APIRET AaffDeInit (void);
 
 enum
 {
-   ERROR_AAFF_MEMALLOC_FAILED   = ERROR_BASE_AAFF + 1,
+   ERROR_AAFF_CANNOT_CLOSE_FILE   = ERROR_BASE_AAFF + 1,
    ERROR_AAFF_CANNOT_CREATE_FILE,
-   ERROR_AAFF_CANNOT_OPEN_FILE,
-   ERROR_AAFF_CANNOT_WRITE_FILE,
-   ERROR_AAFF_CANNOT_CLOSE_FILE,
    ERROR_AAFF_CANNOT_FLUSH_FILE,
-   ERROR_AAFF_CANNOT_READ_SIGNATURE,
-   ERROR_AAFF_CANNOT_READ_HEADER,
-   ERROR_AAFF_CANNOT_READ_FOOTER,
+   ERROR_AAFF_CANNOT_OPEN_FILE,
    ERROR_AAFF_CANNOT_READ_DATA,
    ERROR_AAFF_CANNOT_SEEK,
-   ERROR_AAFF_SECTORSIZE_TOO_BIG,
-   ERROR_AAFF_COMPRESSION_FAILED,
-   ERROR_AAFF_INVALID_SIGNATURE,
-   ERROR_AAFF_INVALID_HEADER,
+   ERROR_AAFF_CANNOT_WRITE_FILE,
    ERROR_AAFF_INVALID_FOOTER,
-   ERROR_AAFF_INVALID_SEGMENTLEN,
-   ERROR_AAFF_INVALID_PAGE_ORDER,
+   ERROR_AAFF_INVALID_HEADER,
    ERROR_AAFF_INVALID_PAGE_ARGUMENT,
-   ERROR_AAFF_INVALID_SEGMENT_NAME,
    ERROR_AAFF_INVALID_PAGE_NUMBER,
+   ERROR_AAFF_INVALID_PAGE_ORDER,
+   ERROR_AAFF_INVALID_SEGMENTLEN,
+   ERROR_AAFF_INVALID_SEGMENT_NAME,
+   ERROR_AAFF_INVALID_SIGNATURE,
+   ERROR_AAFF_MEMALLOC_FAILED,
    ERROR_AAFF_PAGE_LARGER_THAN_BUFFER,
+   ERROR_AAFF_SECTORSIZE_TOO_BIG,
    ERROR_AAFF_UNCOMPRESS_FAILED
 };
 
