@@ -9,7 +9,7 @@
 //  Module:         Standard include file
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -26,9 +26,17 @@
 // You should have received a copy of the GNU General Public License
 // along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
+
+#ifdef LINT_CODE_CHECK
+#include "macros_for_lint.h"
+#endif
+
 #ifndef __COMMON_H__
 #define __COMMON_H__
 
+#define USE_MD5_FROM_OPENSSL
+#define USE_SHA256_FROM_OPENSSL
+
 // GNU C lib definitions
 // ---------------------
 #ifndef _LARGEFILE_SOURCE
