@@ -9,7 +9,7 @@
 //  Module:         Application configuration data
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -195,6 +195,10 @@ static t_ToolCfgSet SetArrColor[] =
    // Name in cfg file          Corresponding value
    // ---------------------------------------------------------
    {  "LocalDevices"          , COLOR_LOCALDEVICES            },
+   {  "AdditionalState1"      , COLOR_ADDITIONALSTATE1        },
+   {  "AdditionalState2"      , COLOR_ADDITIONALSTATE2        },
+   {  "AdditionalState3"      , COLOR_ADDITIONALSTATE3        },
+   {  "AdditionalState4"      , COLOR_ADDITIONALSTATE4        },
    {  "StateIdle"             , COLOR_STATE_IDLE              },
    {  "StateAcquire"          , COLOR_STATE_ACQUIRE           },
    {  "StateAcquirePaused"    , COLOR_STATE_ACQUIRE_PAUSED    },
@@ -209,33 +213,35 @@ static t_ToolCfgSet SetArrColor[] =
 };
 
 
-static t_ToolCfgSet SetArrEwfFormat[] =
-{
-   // Name in cfg file   Corresponding value
-   // ------------------------------------------
-   {  "Encase1",         LIBEWF_FORMAT_ENCASE1},
-   {  "Encase2",         LIBEWF_FORMAT_ENCASE2},
-   {  "Encase3",         LIBEWF_FORMAT_ENCASE3},
-   {  "Encase4",         LIBEWF_FORMAT_ENCASE4},
-   {  "Encase5",         LIBEWF_FORMAT_ENCASE5},
-   {  "Encase6",         LIBEWF_FORMAT_ENCASE6},
-   {  "Smart"  ,         LIBEWF_FORMAT_SMART  },
-   {  "FTK"    ,         LIBEWF_FORMAT_FTK    },
-   {  "Linen5" ,         LIBEWF_FORMAT_LINEN5 },
-   {  "Linen6" ,         LIBEWF_FORMAT_LINEN6 },
-   {  "LVF"    ,         LIBEWF_FORMAT_LVF    },
-   {   NULL    ,         0                    }
-};
-
-
 static t_ToolCfgSet SetArrFormat[] =
 {
    // Name in cfg file   Corresponding value
    // ------------------------------------------
-   {  "DD"     ,         t_File::DD },
-   {  "EWF"    ,         t_File::EWF},
-   {  "AFF"    ,         t_File::AFF},
-   {   NULL    ,         0          }
+   {  "DD"      ,        t_File::DD  },
+   {  "EWF"     ,        t_File::EWF },
+   {  "AFF"     ,        t_File::AAFF},
+   {  "AAFF"    ,        t_File::AAFF},
+   {   NULL     ,        0           }
+};
+
+static t_ToolCfgSet SetArrEwfFormat[] =
+{
+   // Name in cfg file   Corresponding value
+   // ------------------------------------------
+   {  "Encase1" ,         LIBEWF_FORMAT_ENCASE1},
+   {  "Encase2" ,         LIBEWF_FORMAT_ENCASE2},
+   {  "Encase3" ,         LIBEWF_FORMAT_ENCASE3},
+   {  "Encase4" ,         LIBEWF_FORMAT_ENCASE4},
+   {  "Encase5" ,         LIBEWF_FORMAT_ENCASE5},
+   {  "Encase6" ,         LIBEWF_FORMAT_ENCASE6},
+   {  "Smart"   ,         LIBEWF_FORMAT_SMART  },
+   {  "FTK"     ,         LIBEWF_FORMAT_FTK    },
+   {  "Linen5"  ,         LIBEWF_FORMAT_LINEN5 },
+   {  "Linen6"  ,         LIBEWF_FORMAT_LINEN6 },
+   {  "LVF"     ,         LIBEWF_FORMAT_LVF    },
+   {  "Guymager",         t_File::AEWF         }, // AEWF format is a format on its own (for the function calls) and at the same time a
+   {  "AEWF"    ,         t_File::AEWF         }, // subformat of EWF (for the user interface). Looks a bit strange at first sight, but
+   {   NULL     ,         0                    }  // probably is the best compromise.
 };
 
 
@@ -332,10 +338,14 @@ static t_ToolCfgParamDesc CfgParamDescArr[] =
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"AffCompression"                , CFGTYPE_INTEGER, ELT(AffCompression)                ,                0,       1,       9, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"AffMarkBadSectors"             , CFGTYPE_SET    , ELT(AffMarkBadSectors)             ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"SpecialFilenameChars"          , CFGTYPE_STRING , ELT_SIZ(SpecialFilenameChars)      ,                         0,       0, NULL                            }, CFG_FILLUP_FORLINT},
+   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"CalcImageFileMD5"              , CFGTYPE_SET    , ELT(CalcImageFileMD5)              ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
+   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"AvoidEncaseProblems"           , CFGTYPE_SET    , ELT(AvoidEncaseProblems)           ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
 
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"DeviceScanMethod"              , CFGTYPE_SET    , ELT(DeviceScanMethod)              ,                0,       0,       0, INIARR(SetArrDeviceScanMethod)  }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"ScanInterval"                  , CFGTYPE_INTEGER, ELT(ScanInterval                  ),                0,       1, 8640000, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"CommandGetSerialNumber"        , CFGTYPE_STRING , ELT_SIZ(CommandGetSerialNumber    ),                         0,       0, NULL                            }, CFG_FILLUP_FORLINT},
+   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"CommandGetAddStateInfo"        , CFGTYPE_STRING , ELT_SIZ(CommandGetAddStateInfo    ),                         0,       0, NULL                            }, CFG_FILLUP_FORLINT},
+   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"DirectIO"                      , CFGTYPE_SET    , ELT(DirectIO                      ),                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"FifoBlockSizeDD"               , CFGTYPE_INTEGER, ELT(FifoBlockSizeDD               ),                0,       0,99999999, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"FifoBlockSizeEWF"              , CFGTYPE_INTEGER, ELT(FifoBlockSizeEWF              ),                0,       0,99999999, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"FifoBlockSizeAFF"              , CFGTYPE_INTEGER, ELT(FifoBlockSizeAFF              ),                0,       0,99999999, NULL                            }, CFG_FILLUP_FORLINT},
