@@ -7,7 +7,7 @@
 //                  Section Nouvelles Technologies
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -76,6 +76,10 @@ typedef enum
 typedef enum
 {
    COLOR_LOCALDEVICES=0,
+   COLOR_ADDITIONALSTATE1,
+   COLOR_ADDITIONALSTATE2,
+   COLOR_ADDITIONALSTATE3,
+   COLOR_ADDITIONALSTATE4,
    COLOR_STATE_IDLE,
    COLOR_STATE_ACQUIRE,
    COLOR_STATE_ACQUIRE_PAUSED,
@@ -88,7 +92,7 @@ typedef enum
    COLOR_STATE_ABORTED_OTHER,
 
    COLOR_COUNT,
-   COLOR_DEFAULT  // Not really a color, but used for specifying, that a widget should switch to its default color (grey for a button, for instance)
+   COLOR_DEFAULT=-1  // Not really a color, but used for specifying, that a widget should switch to its default color (grey for a button, for instance)
 } t_CfgColor;
 
 typedef enum
@@ -129,6 +133,8 @@ typedef struct
    t_CfgScanMethod      DeviceScanMethod;
    int                  ScanInterval;
    char                 CommandGetSerialNumber[CFG_MAX_PATH_LEN+1];
+   char                 CommandGetAddStateInfo[CFG_MAX_PATH_LEN+1];
+   int                  DirectIO;
 
    int                  DefaultFormat;
    int                  EwfFormat;
@@ -136,6 +142,8 @@ typedef struct
    int                  AffCompression;
    int                  AffMarkBadSectors;
    char                 SpecialFilenameChars[CFG_MAX_MISC_LEN+1];
+   int                  CalcImageFileMD5;
+   int                  AvoidEncaseProblems;
 
    int                  FifoBlockSizeDD;
    int                  FifoBlockSizeEWF;
@@ -144,6 +152,7 @@ typedef struct
    int                  FifoMemoryManager;
    int                  UseSeparateHashThread;
    int                  CompressionThreads;
+
    int                  SignalHandling;
    int                  WriteToDevNull;
    int                  UseMemWatch;
@@ -249,7 +258,7 @@ class t_CfgDlgAcquireField
       QCheckBox            *pCheckBox;
       QComboBox            *pComboBox;
       QLabel               *pLabel;
-};
+}; //lint !e1553
 typedef t_CfgDlgAcquireField       *t_pCfgDlgAcquireField;
 typedef QList<t_pCfgDlgAcquireField> t_CfgDlgAcquireFields;
 typedef t_CfgDlgAcquireFields      *t_pCfgDlgAcquireFields;
@@ -260,7 +269,7 @@ class t_CfgDlgAcquireRule
       QString TriggerFieldName;
       QString DestFieldName;
       QString Value;
-};
+}; //lint !e1553
 typedef t_CfgDlgAcquireRule        *t_pCfgDlgAcquireRule;
 typedef QList<t_pCfgDlgAcquireRule> t_CfgDlgAcquireRules;
 typedef t_CfgDlgAcquireRules       *t_pCfgDlgAcquireRules;
