@@ -267,7 +267,7 @@ Acceptez-vous ces changement?</translation>
     </message>
     <message>
         <source>Hash calculation / verification</source>
-        <translation>Calcul / vérification de valuers hash</translation>
+        <translation>Calcul / vérification de valeurs hash</translation>
     </message>
     <message>
         <source>Guymager cannot write to the directory
@@ -366,7 +366,11 @@ Acceptez-vous ces changement?</translation>
     </message>
     <message>
         <source>The split size for the configured EWF format must be in the range %1 - %2 MiB.For bigger files, switch to Encase6 format or later (see Guymager configuration file, parameter EwfFormat).</source>
-        <translation>La taille des fragments pour le format EWF configuré doit être comprise entre %1 - %2 MiB. Des fichiers plus grands sont possibles à partir du format Encase6 (voir fichier de configuration, paramètre EwfFormat).</translation>
+        <translation type="obsolete">La taille des fragments pour le format EWF configuré doit être comprise entre %1 - %2 MiB. Des fichiers plus grands sont possibles à partir du format Encase6 (voir fichier de configuration, paramètre EwfFormat).</translation>
+    </message>
+    <message>
+        <source>The split size for the configured EWF format must be in the range %1 - %2 MiB. For bigger files, switch to Encase6 format or later (see Guymager configuration file, parameter EwfFormat).</source>
+        <translation>La taille des fragments pour le format EWF configuré doit être comprise entre %1 et %2 MiB. Des fichiers plus grands sont possibles à partir du format Encase6 (voir fichier de configuration, paramètre EwfFormat).</translation>
     </message>
 </context>
 <context>
@@ -500,6 +504,10 @@ Guymager set terminera automatiquement dans %1 secondes.</translation>
         <comment>Display that verification is off</comment>
         <translation>non</translation>
     </message>
+    <message>
+        <source>Additional state info</source>
+        <translation>Informations d&apos;état suppl.</translation>
+    </message>
 </context>
 <context>
     <name>t_MainWindow</name>
@@ -598,7 +606,7 @@ Compilation timestamp: %2
 Compiled with gcc %3
 Using libewf %4
 Using libguytools %5</source>
-        <translation>GUYMAGER est un outil d&apos;acquisition forensique basé sur Linux
+        <translation type="obsolete">GUYMAGER est un outil d&apos;acquisition forensique basé sur Linux
 
 Version: %1
 Date et heure de compilation: %2
@@ -715,6 +723,34 @@ Utilise libguytools %5</translation>
         <source>Exit after acquisitions completed</source>
         <translation>Terminer Guymager à la fin des acquisitions</translation>
     </message>
+    <message>
+        <source>Aborted - Info file write error</source>
+        <translation>Abandon - accès fichier info échoué</translation>
+    </message>
+    <message>
+        <source>Aborted - Acquisition start failure</source>
+        <translation>Abandon - Démarrage échoué</translation>
+    </message>
+    <message>
+        <source>(not used as Guymager currently is configured to use its own EWF module)</source>
+        <translation>(inactif, Guymager est configuré d&apos;utiliser son propre module EWF)</translation>
+    </message>
+    <message>
+        <source>GUYMAGER is a Linux-based forensic imaging tool
+
+Version: %1
+Compilation timestamp: %2
+Compiled with gcc %3
+Linked with libewf %4 %5
+Linked with libguytools %6</source>
+        <translation>GUYMAGER est un outil d&apos;acquisition forensique basé sur Linux
+
+Version: %1
+Date et heure de compilation: %2
+Compilé avec gcc %3
+Utilise libewf %4 %5
+Utilise libguytools %6</translation>
+    </message>
 </context>
 <context>
     <name>t_Table</name>
@@ -1034,6 +1070,27 @@ Utilise libguytools %5</translation>
         <source>Device used for cloning - cannot be acquired</source>
         <translation>Occupé par opération de clonage - ne peut être  acquéré</translation>
     </message>
+    <message>
+        <source>Generated image files and their MD5 hashes</source>
+        <comment>Info file</comment>
+        <translation>Fichiers image et leur valeurs hash MD5</translation>
+    </message>
+    <message>
+        <source>No MD5 hashes available (the select EWF format does not support image file hashes)</source>
+        <translation>Valeurs hashs MD5 non disponibles (le format EWF choisi ne supporte pas le calcul des valeurs hash des fichiers image)</translation>
+    </message>
+    <message>
+        <source>No MD5 hashes available (image verification checkbox was switched off)</source>
+        <translation>Valeurs hashs MD5 non disponibles (l&apos;option pour la vérification de l&apos;image n&apos;était pas activée)</translation>
+    </message>
+    <message>
+        <source>No MD5 hashes available (configuration parameter CalcImageFileMD5 is off)</source>
+        <translation>Valeurs hashs MD5 non disponibles (le paramètre de configuration CalcImageFileMD5 est désactivé)</translation>
+    </message>
+    <message>
+        <source>Cannot be acquired - see additional state info field</source>
+        <translation>Ne peut être acquéré - voir informations d&apos;état suppl.</translation>
+    </message>
 </context>
 <context>
     <name>t_ThreadScan</name>
