@@ -9,7 +9,7 @@
 //  Module:         Hash wrapper functions for uniform hash interface
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -31,10 +31,24 @@
 #include <QtCore>
 #include "hash.h"
 
+#ifdef USE_MD5_FROM_OPENSSL
+#include <openssl/md5.h>
+#endif
+
+#ifdef USE_SHA256_FROM_OPENSSL
+#include <openssl/sha.h>
+#endif
+
 // ------------------------------------
 //             MD5 Functions
 // ------------------------------------
 
+APIRET HashMD5Clear (t_pHashContextMD5 pContext)
+{
+   memset (pContext, 0, sizeof(t_HashContextMD5));
+   return NO_ERROR;
+}
+
 APIRET HashMD5Init (t_pHashContextMD5 pContext)
 {
    #ifdef USE_MD5_FROM_OPENSSL
@@ -85,21 +99,39 @@ bool HashMD5Match (t_pHashMD5Digest pDigest1, t_pHashMD5Digest pDigest2)
 //           SHA256 Functions
 // ------------------------------------
 
+APIRET HashSHA256Clear (t_pHashContextSHA256 pContext)
+{
+   memset (pContext, 0, sizeof(t_HashContextSHA256));
+   return NO_ERROR;
+}
+
 APIRET HashSHA256Init (t_pHashContextSHA256 pContext)
 {
-   sha256_starts (pContext);
+   #ifdef USE_SHA256_FROM_OPENSSL
+      (void) SHA256_Init (pContext);
+   #else
+      sha256_starts (pContext);
+   #endif
    return NO_ERROR;
 }
 
 APIRET HashSHA256Append (t_pHashContextSHA256 pContext, void *pData, int DataLen)
 {
-   sha256_update (pContext, (unsigned char *)pData, DataLen);
+   #ifdef USE_SHA256_FROM_OPENSSL
+      (void) SHA256_Update (pContext, pData, (unsigned long) DataLen);
+   #else
+      sha256_update (pContext, (unsigned char *)pData, DataLen);
+   #endif
    return NO_ERROR;
 }
 
 APIRET HashSHA256Digest (t_pHashContextSHA256 pContext, t_pHashSHA256Digest pDigest)
 {
-   sha256_finish (pContext, (unsigned char *)pDigest);
+   #ifdef USE_SHA256_FROM_OPENSSL
+      (void) SHA256_Final (&pDigest->Buff[0], pContext);
+   #else
+      sha256_finish (pContext, (unsigned char *)pDigest);
+   #endif
    return NO_ERROR;
 }
 
