@@ -9,7 +9,7 @@
 //  Module:         Hash wrapper functions for uniform hash interface
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -33,18 +33,24 @@
    #include "common.h"
 #endif
 
-#ifdef USE_MD5_FROM_OPENSSL   // Only for test purposes; It is strongly recommended to use the local md5 functions in order to avoid an OpenSSL dependency
+#ifdef USE_MD5_FROM_OPENSSL
    #include <openssl/md5.h>
    typedef MD5_CTX t_HashContextMD5;
 #else
    #include "md5.h"
    typedef md5_state_t t_HashContextMD5;
 #endif
-typedef t_HashContextMD5 *t_pHashContextMD5;
 
-#include "sha256.h"
-typedef sha256_context  t_HashContextSHA256;
-typedef sha256_context *t_pHashContextSHA256;
+#ifdef USE_SHA256_FROM_OPENSSL
+   #include <openssl/sha.h>
+   typedef SHA256_CTX t_HashContextSHA256;
+#else
+   #include "sha256.h"
+   typedef sha256_context  t_HashContextSHA256;
+#endif
+
+typedef t_HashContextMD5    *t_pHashContextMD5;
+typedef t_HashContextSHA256 *t_pHashContextSHA256;
 
 
 
@@ -63,12 +69,14 @@ typedef struct
    unsigned char Buff[HASH_SHA256_DIGEST_LENGTH];
 } t_HashSHA256Digest, *t_pHashSHA256Digest;
 
+APIRET HashMD5Clear     (t_pHashContextMD5 pContext);
 APIRET HashMD5Init      (t_pHashContextMD5 pContext);
 APIRET HashMD5Append    (t_pHashContextMD5 pContext, const void *pData, int DataLen);
 APIRET HashMD5Digest    (t_pHashContextMD5 pContext, t_pHashMD5Digest pDigest);
 APIRET HashMD5DigestStr (t_pHashMD5Digest pDigest, QString &Str);
 bool   HashMD5Match     (t_pHashMD5Digest pDigest1, t_pHashMD5Digest pDigest2);
 
+APIRET HashSHA256Clear     (t_pHashContextSHA256 pContext);
 APIRET HashSHA256Init      (t_pHashContextSHA256 pContext);
 APIRET HashSHA256Append    (t_pHashContextSHA256 pContext, void *pData, int DataLen);
 APIRET HashSHA256Digest    (t_pHashContextSHA256 pContext, t_pHashSHA256Digest pDigest);
