@@ -11,7 +11,7 @@
 //                  deinitialisation.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -51,6 +51,7 @@
 #include "config.h"
 #include "device.h"
 #include "qtutil.h"
+#include "util.h"
 #include "hash.h"
 #include "aaff.h"
 #include "aewf.h"
@@ -441,6 +442,7 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
    // ----------------------------------------------------------------------
    MainLocal.CriticalInitDone = true;
 
+   CHK (UtilInit    ())
    CHK (HashInit    ())
    CHK (QtUtilInit  ())
    CHK (t_File::Init())
@@ -497,6 +499,7 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
    CHK (AaffDeInit  ())
    CHK (QtUtilDeInit())
    CHK (HashDeInit  ())
+   CHK (UtilDeInit  ())
    CHK (CfgDeInit   ())
 
    delete MainLocal.pApp;
