@@ -790,7 +790,10 @@ APIRET t_MediaInfo::QueryDevice (const char *pLinuxDevice)
 
       CHK (MediaGetSectorCount (File, pLinuxDevice, SupportsLBA48, &SectorCountDeviceConfigIdentify, &SectorCountReadNativeMax))
 
-      LOG_INFO ("Identification of %s: [%s] [%s] %llu / %llu / %llu", pLinuxDevice, QSTR_TO_PSZ(Model), QSTR_TO_PSZ(SerialNumber), SectorCountIdentifyDevice, SectorCountReadNativeMax, SectorCountDeviceConfigIdentify)
+      LOG_INFO ("%s model and serial number: [%s] [%s]", pLinuxDevice, QSTR_TO_PSZ(Model), QSTR_TO_PSZ(SerialNumber))
+      LOG_INFO ("%s sectors->IdentifyDevice: %llu"     , pLinuxDevice, SectorCountIdentifyDevice      )
+      LOG_INFO ("%s sectors->ReadNativeMax : %llu"     , pLinuxDevice, SectorCountReadNativeMax       )
+      LOG_INFO ("%s sectors->ConfigIdentify: %llu"     , pLinuxDevice, SectorCountDeviceConfigIdentify)
 //      printf ("\n[%s] [%s] [%s]", pLinuxDevice, QSTR_TO_PSZ(Model), QSTR_TO_PSZ(SerialNumber));
 //      printf ("\nIdentify       %16llu %16llX", SectorCountIdentifyDevice      , SectorCountIdentifyDevice      );
 //      printf ("\nNative         %16llu %16llX", SectorCountReadNativeMax       , SectorCountReadNativeMax       );
