@@ -9,7 +9,7 @@
 //  Module:         Central application error code managment
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -30,7 +30,9 @@
 #define __MODULES_H__
 
 #define CREATE_MODULE_IDS(ModulName, x) \
-   const int ERROR_BASE_ ## ModulName = ERROR_BASE_GUYMAGER_MAIN +  x * ID_OFFSET_SUBSUBPROJECT;
+   /*lint -save -e835 -e845*/           \
+   const unsigned int ERROR_BASE_ ## ModulName = ERROR_BASE_GUYMAGER_MAIN +  x * ID_OFFSET_SUBSUBPROJECT; \
+   /*lint -restore*/
 
 
 CREATE_MODULE_IDS (MAIN           ,  0)
@@ -68,8 +70,8 @@ CREATE_MODULE_IDS (QT             , 99)
 
 enum
 {
-   ERROR_QT_UNSUCCESSFUL = ERROR_BASE_QT + 1
-};
+   ERROR_QT_UNSUCCESSFUL = (ERROR_BASE_QT + 1)
+}; //lint !e30
 
 
 #endif
