@@ -9,7 +9,7 @@
 //  Module:         Different Qt utility functions
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -356,13 +356,15 @@ APIRET QtUtilProcessCommand (const QString &Command, QString &StandardOutput)
    }
    if (qtrc)
    {
-//      StandardOutput  = Process.readAllStandardOutput();
-      StandardOutput  = Process.readAll();
+//    StandardOutput  = Process.readAllStandardOutput();
+//    StandardOutput  = Process.readAll();
+      StandardOutput  = QString::fromUtf8 (Process.readAll().constData());
+      
       StrState = "Done";
    }
    else
    {
-      LOG_INFO ("Killing command because of timeout [%s] in state %s", QSTR_TO_PSZ(Command), QSTR_TO_PSZ(StrState))
+      LOG_INFO ("Killing command because of timeout or non-existence [%s] in state %s", QSTR_TO_PSZ(Command), QSTR_TO_PSZ(StrState))
       Process.kill();
       return ERROR_QTUTIL_COMMAND_TIMEOUT;
    }
