@@ -9,7 +9,7 @@
 //  Module:         Thread for scanning the connected devices
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -44,7 +44,11 @@
 //         Constants
 // --------------------------
 
-// #define DEBUG_SERNR
+//#define DEBUG_SERNR
+
+#ifdef DEBUG_SERNR
+   #warning "DEBUG_SERNR must be switched off for final release"
+#endif
 
 const int THREADSCAN_WAIT_MAX         = 10000;
 const int THREADSCAN_WAIT_GRANULARITY = 100;
@@ -52,6 +56,16 @@ const int THREADSCAN_WAIT_GRANULARITY = 100;
 const QString ThreadScanLibPartedSearchDirs    = "/lib,/usr/lib,/usr/local/lib";       // Separate directories by commas
 const QString ThreadScanLibPartedSearchPattern = "libparted*.so*";
 
+
+// -----------------------------------
+//        Worker base class
+// -----------------------------------
+
+t_ThreadScanWorker::t_ThreadScanWorker (t_ThreadScan *pThreadScan)
+{
+   poThreadScan = pThreadScan;
+}
+
 // -----------------------------------
 //  Utility functions used by workers
 // -----------------------------------
@@ -73,6 +87,33 @@ static APIRET ThreadScanMarkLocalDevices (const t_pDeviceList pDeviceList)
    return NO_ERROR;
 }
 
+static APIRET ThreadScanGetAddStateInfo (t_pDevice pDevice)
+{
+   QString     Command;
+   APIRET      rc;
+   QString     StateInfo;
+   QStringList StateInfoList;
+
+   Command = CONFIG (CommandGetAddStateInfo);
+   if (!Command.isEmpty())
+   {
+      Command.replace ("%dev", pDevice->LinuxDevice, Qt::CaseInsensitive);
+   
+      rc = QtUtilProcessCommand (Command, StateInfo);
+      if (rc == ERROR_QTUTIL_COMMAND_TIMEOUT)
+      {
+         LOG_INFO ("Timeout while trying to get additional state information for %s, additional state info will be ignored", QSTR_TO_PSZ(pDevice->LinuxDevice))
+         return NO_ERROR;
+      }
+      CHK(rc)
+      StateInfoList = StateInfo.split("\n");
+      if (StateInfoList.count() > 0) pDevice->AddStateInfo.Info          =  StateInfoList[0].trimmed();
+      if (StateInfoList.count() > 1) pDevice->AddStateInfo.CanBeAcquired = (StateInfoList[1].trimmed() != "0");
+      if (StateInfoList.count() > 2) pDevice->AddStateInfo.Color         =  StateInfoList[2].trimmed().toInt();
+   }
+   return NO_ERROR;
+}
+
 // --------------------------
 //  t_ThreadScanWorkerParted
 // --------------------------
@@ -152,7 +193,8 @@ static APIRET ThreadScanFindLibparted (QStringList &Files)
 }
 
 
-t_ThreadScanWorkerParted::t_ThreadScanWorkerParted (APIRET &rc)
+t_ThreadScanWorkerParted::t_ThreadScanWorkerParted (t_ThreadScan *pThreadScan, APIRET &rc)
+   :t_ThreadScanWorker (pThreadScan)
 {
    QStringList   FilenameList;
    QString       Filename;
@@ -232,6 +274,7 @@ void t_ThreadScanWorkerParted::SlotRescan (void)
 
    emit (SignalScanStarted ());
    LOG_INFO ("Rescanning devices")
+   CHK_EXIT (poThreadScan->SetDebugMessage ("SlotRescan called (parted)"))
 
    pOwn->pFnLibPartedProbeAll ();
 
@@ -260,11 +303,13 @@ void t_ThreadScanWorkerParted::SlotRescan (void)
       pDevice = pDeviceList->AppendNew (SerialNumber, pPedDev);
       if (pDevice->LinuxDevice.startsWith ("/dev/fd",  Qt::CaseSensitive)) // Not a good way for checking this, but I don't know how to extract the information from
          pDevice->Removable = true;                                        // PedDevice. BTW, this won't work with other removable devices, for instance memory sticks.
+      CHK_EXIT (ThreadScanGetAddStateInfo (pDevice));
    }
    pOwn->pFnLibPartedFreeAll ();
    CHK_EXIT (ThreadScanMarkLocalDevices (pDeviceList))
 
    emit (SignalScanFinished (pDeviceList));
+   CHK_EXIT (poThreadScan->SetDebugMessage ("Leaving SlotRescan (parted)"))
 }
 
 // --------------------------
@@ -301,7 +346,8 @@ class t_ThreadScanWorkerHALLocal
 };
 
 
-t_ThreadScanWorkerHAL::t_ThreadScanWorkerHAL (APIRET &rc)
+t_ThreadScanWorkerHAL::t_ThreadScanWorkerHAL (t_ThreadScan *pThreadScan, APIRET &rc)
+   :t_ThreadScanWorker (pThreadScan)
 {
    pOwn = new t_ThreadScanWorkerHALLocal;
    if (!pOwn->pDBusConnection->isConnected())
@@ -421,6 +467,8 @@ void t_ThreadScanWorkerHAL::SlotRescan (void)
    QVariant         RemovableAvailable;
    QString          Model;
 
+   CHK_EXIT (poThreadScan->SetDebugMessage ("SlotRescan called (HAL)"))
+
    if (thread() != QThread::currentThread())
       CHK_EXIT (ERROR_THREADSCAN_CALLED_FROM_WRONG_THREAD) // As Qt's DBus system is quite sensible to this kind of
                                                            // mistake (resulting in many QTimer "cannot be stopped/started
@@ -506,11 +554,13 @@ void t_ThreadScanWorkerHAL::SlotRescan (void)
          pDevice->Removable = Removable.toBool();
 
          CHK_EXIT (pDevice->MediaInfo.QueryDevice(QSTR_TO_PSZ(LinuxDevice.toString())))
+         CHK_EXIT (ThreadScanGetAddStateInfo (pDevice));
       }
    }
    CHK_EXIT (ThreadScanMarkLocalDevices (pDeviceList))
    LOG_INFO ("Device scan finished")
 
+   CHK_EXIT (poThreadScan->SetDebugMessage ("Leaving SlotRescan (HAL)"))
    emit (SignalScanFinished (pDeviceList));
 }
 
@@ -566,7 +616,8 @@ class t_ThreadScanWorkerDevKitLocal
 };
 
 
-t_ThreadScanWorkerDevKit::t_ThreadScanWorkerDevKit (APIRET &rc)
+t_ThreadScanWorkerDevKit::t_ThreadScanWorkerDevKit (t_ThreadScan *pThreadScan, APIRET &rc)
+   :t_ThreadScanWorker (pThreadScan)
 {
    QDBusMessage Message;
 
@@ -688,6 +739,8 @@ void t_ThreadScanWorkerDevKit::SlotRescan (void)
    QVariant        RemovableAvailable;
    QString         Model;
 
+   CHK_EXIT (poThreadScan->SetDebugMessage ("SlotRescan called (DevKit)"))
+
    if (thread() != QThread::currentThread())
       CHK_EXIT (ERROR_THREADSCAN_CALLED_FROM_WRONG_THREAD) // As Qt's DBus system is quite sensible to this kind of
                                                            // mistake (resulting in many QTimer "cannot be stopped/started
@@ -763,11 +816,14 @@ void t_ThreadScanWorkerDevKit::SlotRescan (void)
          pDevice = pDeviceList->AppendNew (SerialNumber.toString(), LinuxDevice.toString(), Model,
                                            BlockSize.toULongLong(), BlockSize.toULongLong(), Size.toULongLong());
          pDevice->Removable = Removable.toBool();
+         CHK_EXIT (pDevice->MediaInfo.QueryDevice(QSTR_TO_PSZ(LinuxDevice.toString())))
+         CHK_EXIT (ThreadScanGetAddStateInfo (pDevice));
       }
    }
    CHK_EXIT (ThreadScanMarkLocalDevices (pDeviceList))
    LOG_INFO ("Device scan finished")
 
+   CHK_EXIT (poThreadScan->SetDebugMessage ("Leaving SlotRescan (DevKit)"))
    emit (SignalScanFinished (pDeviceList));
 }
 
@@ -871,6 +927,8 @@ APIRET t_ThreadScan::Start (t_ThreadScanWorker **ppWorker)
    int  Tries;
    bool Ok;
 
+   CHK_EXIT (SetDebugMessage ("Launch thread"))
+
    #define LAUNCH_WORKER                                                                    \
    {                                                                                        \
       *ppWorker   = NULL;                                                                   \
@@ -912,6 +970,8 @@ APIRET t_ThreadScan::Start (t_ThreadScanWorker **ppWorker)
    if (*ppWorker == NULL)
       CHK (ERROR_THREADSCAN_NOT_STARTED)
 
+   CHK_EXIT (SetDebugMessage ("Thread launched"))
+
    return NO_ERROR;
 }
 
@@ -943,14 +1003,15 @@ void t_ThreadScan::run (void)
    int      rc;
 
    LOG_INFO ("Thread Scan started")
+   CHK_EXIT (SetDebugMessage ("Start run function"))
 
    switch (CONFIG(DeviceScanMethod))
    {
       // We have to create the following object as we want to work with signals and slots in this new thread.
       // t_ThreadScan itself belongs to the main thread and thus can't be used for signals and slots.
-      case SCANMETHOD_DBUSDEVKIT: *ppoWorker = new t_ThreadScanWorkerDevKit (oWorkerRc); break;
-      case SCANMETHOD_DBUSHAL:    *ppoWorker = new t_ThreadScanWorkerHAL    (oWorkerRc); break;
-      case SCANMETHOD_LIBPARTED:  *ppoWorker = new t_ThreadScanWorkerParted (oWorkerRc); break;
+      case SCANMETHOD_DBUSDEVKIT: *ppoWorker = new t_ThreadScanWorkerDevKit (this, oWorkerRc); break;
+      case SCANMETHOD_DBUSHAL:    *ppoWorker = new t_ThreadScanWorkerHAL    (this, oWorkerRc); break;
+      case SCANMETHOD_LIBPARTED:  *ppoWorker = new t_ThreadScanWorkerParted (this, oWorkerRc); break;
       default: CHK_EXIT (ERROR_THREADSCAN_INVALID_SCAN_METHOD)
    }
    if (oWorkerRc)
@@ -968,6 +1029,7 @@ void t_ThreadScan::run (void)
       CHK_EXIT (ERROR_THREADSCAN_EXITCODE_NONZERO)
    }
    LOG_INFO ("Thread Scan ended")
+   CHK_EXIT (SetDebugMessage ("Exit run function"))
    delete pTimer;
    delete *ppoWorker;
    *ppoWorker = NULL;
