@@ -9,7 +9,7 @@
 //  Module:         Thread for writing data.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -46,26 +46,38 @@
 #include "threadwrite.h"
 
 
-const unsigned long THREADWRITE_WAIT_FOR_HANDLE_GRANULARITY =   100; // ms
+const unsigned long THREADWRITE_WAIT_FOR_HANDLE_GRANULARITY =   500; // ms
 const unsigned long THREADWRITE_WAIT_FOR_HANDLE             = 30000; // ms
 const unsigned long THREADWRITE_SLOWDOWN_SLEEP              =   700; // ms
 
+const unsigned long THREADWRITE_ENCASE_MAXLEN_IMAGERVERSION = 11;
+const unsigned long THREADWRITE_ENCASE_MAXLEN_OSVERSION     = 23;
+
+
 class t_OutputFile
 {
    public:
-      virtual        ~t_OutputFile  (void) {};
+                      t_OutputFile  (t_ThreadWrite *pThreadWrite) :poThreadWrite(pThreadWrite) {}
+      virtual        ~t_OutputFile  () {}
       virtual APIRET  Open          (t_pDevice pDevice, bool Verify) = 0;
       virtual APIRET  Write         (t_pFifoBlock pFifoBlock)        = 0;
-      virtual APIRET  Verify        (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, quint64 *pPos) = 0;
+      virtual APIRET  Verify        (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, t_Device::t_ImageFileHashList &ImageFileHashList, quint64 *pPos) = 0;
       virtual APIRET  Close         (void)                           = 0;
       virtual bool    Opened        (void)                           = 0;
       virtual void *  GetFileHandle (void)                           = 0;
+
+      t_ThreadWrite *poThreadWrite;
 };
 
+// ------------------------------------------------------------------------------------------------
+//                                            DD FORMAT
+// ------------------------------------------------------------------------------------------------
+
 class t_OutputFileDD: public t_OutputFile
 {
    public:
-      t_OutputFileDD (void) :
+      t_OutputFileDD (t_ThreadWrite *pThreadWrite) :
+          t_OutputFile         (pThreadWrite),
           oFile                (-1),
           oLastCheckT          (0),
          poVerifyBuff          (NULL),
@@ -137,7 +149,7 @@ class t_OutputFileDD: public t_OutputFile
                oFile = open64 (QSTR_TO_PSZ (Filename), oFileFlags, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
                if (oFile < 0)
                {
-                  LOG_ERROR ("open64 on %s failed, errno=%d '%s'", QSTR_TO_PSZ (oFilename), errno, ToolErrorTranslateErrno (errno))
+                  LOG_ERROR ("[%s] open64 on %s failed, errno=%d '%s'", QSTR_TO_PSZ (poDevice->LinuxDevice), QSTR_TO_PSZ (oFilename), errno, ToolErrorTranslateErrno (errno))
                   return ERROR_THREADWRITE_OPEN_FAILED;
                }
             }
@@ -145,7 +157,7 @@ class t_OutputFileDD: public t_OutputFile
             Written = write (oFile, &pFifoBlock->Buffer[Offset], GETMIN(RemainingData, oRemainingFileSize));
             if (Written < 0)
             {
-               LOG_ERROR ("write failed, oFile %d, errno=%d '%s'", oFile, errno, ToolErrorTranslateErrno (errno))
+               LOG_ERROR ("[%s] Write failed, oFile %d, errno=%d '%s'", QSTR_TO_PSZ (poDevice->LinuxDevice), oFile, errno, ToolErrorTranslateErrno (errno))
                return ERROR_THREADWRITE_WRITE_FAILED;
             }
             RemainingData      -= Written;
@@ -169,7 +181,7 @@ class t_OutputFileDD: public t_OutputFile
                   Error = (fclose (pFile) != 0);
                if (Error)
                {
-                  LOG_ERROR ("Check for destination clone %s failed (%d '%s')", QSTR_TO_PSZ (oFilename), errno, ToolErrorTranslateErrno (errno))
+                  LOG_ERROR ("[%s] Check for destination clone %s failed (%d '%s')", QSTR_TO_PSZ (poDevice->LinuxDevice), QSTR_TO_PSZ (oFilename), errno, ToolErrorTranslateErrno (errno))
                   return ERROR_THREADWRITE_WRITE_FAILED;
                }
                oLastCheckT = NowT;
@@ -179,9 +191,10 @@ class t_OutputFileDD: public t_OutputFile
          return NO_ERROR;
       }
 
-      APIRET Verify (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, quint64 *pPos)
+      APIRET Verify (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, t_Device::t_ImageFileHashList &ImageFileHashList, quint64 *pPos)
       {
-         ssize_t Read;
+         t_HashMD5Digest MD5Digest;
+         ssize_t         Read;
 
          if (*pPos == 0)       // First call of this function?
          {
@@ -200,23 +213,36 @@ class t_OutputFileDD: public t_OutputFile
             if (!Err) Err = (lseek64 (oFile, 0, SEEK_SET) != 0);
             if (Err)
             {
-               LOG_ERROR ("Opening %s failed, errno=%d '%s'", QSTR_TO_PSZ (oFilename), errno, ToolErrorTranslateErrno (errno))
+               LOG_ERROR ("[%s] Opening %s failed, errno=%d '%s'", QSTR_TO_PSZ (poDevice->LinuxDevice), QSTR_TO_PSZ (oFilename), errno, ToolErrorTranslateErrno (errno))
                return ERROR_THREADWRITE_OPEN_FAILED;
             }
             oRemainingFileSize = (t_uint64) FileSize;
+            if (CONFIG(CalcImageFileMD5))
+               CHK (HashMD5Init (&oCurrentHashContextMD5))
          }
 
          Read = read (oFile, (char*)poVerifyBuff, GETMIN(poDevice->FifoBlockSize, oRemainingFileSize));
          if (Read < 0)
          {
             QString Filename = oFilenameList[oCurrentVerifyFileNr-1];
-            LOG_ERROR ("read failed, oFile %d [%s], seek pos %llu, errno %d '%s'", oFile, QSTR_TO_PSZ (Filename), *pPos, errno, ToolErrorTranslateErrno (errno))
+            LOG_ERROR ("[%s] Read failed, oFile %d [%s], seek pos %llu, errno %d '%s'", QSTR_TO_PSZ (poDevice->LinuxDevice), oFile, QSTR_TO_PSZ (Filename), *pPos, errno, ToolErrorTranslateErrno (errno))
             return ERROR_THREADWRITE_VERIFY_FAILED;
          }
          oRemainingFileSize -= Read;
+         if (CONFIG(CalcImageFileMD5))
+            CHK_EXIT (HashMD5Append (&oCurrentHashContextMD5, poVerifyBuff, Read))
 
          if (oRemainingFileSize == 0)
+         {
+            bool MD5Valid = false;
+            if (CONFIG(CalcImageFileMD5))
+            {
+               CHK (HashMD5Digest (&oCurrentHashContextMD5, &MD5Digest))
+               MD5Valid = true;
+            }
+            ImageFileHashList.append (new t_Device::t_ImageFileHash (oFilenameList[oCurrentVerifyFileNr-1], &MD5Digest, MD5Valid));
             Close0 (false);
+         }
 
          if (pHashContextMD5)    CHK_EXIT (HashMD5Append    (pHashContextMD5   , poVerifyBuff, Read))
          if (pHashContextSHA256) CHK_EXIT (HashSHA256Append (pHashContextSHA256, poVerifyBuff, Read))
@@ -237,7 +263,7 @@ class t_OutputFileDD: public t_OutputFile
          oFile = -1;
          if (Res != 0)
          {
-            LOG_ERROR ("close failed, errno=%d '%s'", errno, ToolErrorTranslateErrno (errno))
+            LOG_ERROR ("[%s] Close failed, errno=%d '%s'", QSTR_TO_PSZ (poDevice->LinuxDevice), errno, ToolErrorTranslateErrno (errno))
             return ERROR_THREADWRITE_CLOSE_FAILED;
          }
 
@@ -267,25 +293,30 @@ class t_OutputFileDD: public t_OutputFile
       }
 
    private:
-      QStringList oFilenameList;
-      int         oFile;
-      time_t      oLastCheckT; // Only used when cloning, see above
-      QString     oFilename;
-      void      *poVerifyBuff;
-      t_pDevice  poDevice;
-      int         oSplitNr;
-      int         oSplitNrDecimals;
-      int         oFileFlags;
-      t_uint64    oRemainingFileSize;
-      int         oCurrentVerifyFileNr;
+      QStringList       oFilenameList;
+      int               oFile;
+      time_t            oLastCheckT; // Only used when cloning, see above
+      QString           oFilename;
+      void            *poVerifyBuff;
+      t_pDevice        poDevice;
+      int               oSplitNr;
+      int               oSplitNrDecimals;
+      int               oFileFlags;
+      t_uint64          oRemainingFileSize;
+      int               oCurrentVerifyFileNr;
+      t_HashContextMD5  oCurrentHashContextMD5;
 };
 
+// ------------------------------------------------------------------------------------------------
+//                                           EWF FORMAT
+// ------------------------------------------------------------------------------------------------
+
 #define CHK_LIBEWF(Fn)                                                 \
 {                                                                      \
    int rclibewf = (Fn);                                                \
    if (rclibewf != 1)                                                  \
    {                                                                   \
-      LOG_ERROR ("Error in libewf function: %s, rc=%d", #Fn, rclibewf) \
+      LOG_ERROR ("[%s] Error in libewf function: %s, rc=%d", QSTR_TO_PSZ (poDevice->LinuxDevice), #Fn, rclibewf) \
       return ERROR_THREADWRITE_LIBEWF_FAILED;                          \
    }                                                                   \
 }
@@ -293,7 +324,8 @@ class t_OutputFileDD: public t_OutputFile
 class t_OutputFileEWF: public t_OutputFile
 {
    public:
-      t_OutputFileEWF (void)
+      t_OutputFileEWF (t_ThreadWrite *pThreadWrite):
+          t_OutputFile (pThreadWrite)
       {
          poFile                  = NULL;
          poDevice                = NULL;   //lint -esym(613,t_OutputFileEWF::poDevice)  Prevent lint from telling us about possible null pointers in the following code
@@ -342,10 +374,13 @@ class t_OutputFileEWF: public t_OutputFile
                poImageFilenameArr[i] = (char *) malloc (strlen(pAsciiFileName)+1);
                strcpy (poImageFilenameArr[i], pAsciiFileName);
             }
+
+            CHK_EXIT (poThreadWrite->SetDebugMessage ("Verification: Calling libewf_open"))
             pFile = libewf_open (poImageFilenameArr, oImageFileCount, libewf_get_flags_read());
+            CHK_EXIT (poThreadWrite->SetDebugMessage ("Verification: Returning from libewf_open"))
             if (pFile == NULL)
             {
-               LOG_INFO ("Error while reopening EWF for verification. The files are:")
+               LOG_INFO ("[%s] Error while reopening EWF for verification. The files are:", QSTR_TO_PSZ (poDevice->LinuxDevice))
                for (i=0; i<oImageFileCount; i++)
                   LOG_INFO ("%s", poImageFilenameArr[i])
                return ERROR_THREADWRITE_OPEN_FAILED;
@@ -360,16 +395,16 @@ class t_OutputFileEWF: public t_OutputFile
                return ERROR_THREADWRITE_OPEN_FAILED;
 
             #define STR_AND_LEN(QStr) QStr.toAscii().data(), strlen(QStr.toAscii().data())
-            libewf_set_notify_values(stderr, 1);
 
             CHK_LIBEWF (libewf_set_format             (pFile, (uint8_t) CONFIG (EwfFormat)))
             CHK_LIBEWF (libewf_set_media_size         (pFile, pDevice->Size))
             CHK_LIBEWF (libewf_set_bytes_per_sector   (pFile, (unsigned int) pDevice->SectorSize))
-            CHK_LIBEWF (libewf_set_sectors_per_chunk  (pFile, 64))
+            CHK_LIBEWF (libewf_set_sectors_per_chunk  (pFile, pDevice->FifoBlockSize / pDevice->SectorSize))
             CHK_LIBEWF (libewf_set_segment_file_size  (pFile, pDevice->Acquisition.SplitFileSize))
-            CHK_LIBEWF (libewf_set_compression_values (pFile, CONFIG (EwfCompression), 1))
+            CHK_LIBEWF (libewf_set_compression_values (pFile, CONFIG (EwfCompression), 0)) // last parameter must be set to 0, else, only empty-block compression is done
             CHK_LIBEWF (libewf_set_media_type         (pFile, pDevice->Removable ? LIBEWF_MEDIA_TYPE_REMOVABLE : LIBEWF_MEDIA_TYPE_FIXED))
             CHK_LIBEWF (libewf_set_volume_type        (pFile, LIBEWF_VOLUME_TYPE_PHYSICAL))
+            CHK_LIBEWF (libewf_set_error_granularity  (pFile, 1))
 
             CHK_LIBEWF (libewf_set_header_value (pFile, (char *)"case_number"    , STR_AND_LEN(pDevice->Acquisition.CaseNumber    )))
             CHK_LIBEWF (libewf_set_header_value (pFile, (char *)"description"    , STR_AND_LEN(pDevice->Acquisition.Description   )))
@@ -379,6 +414,12 @@ class t_OutputFileEWF: public t_OutputFile
 
             CHK (ToolSysInfoUname (Uname))
             GuymagerVersion = QString("guymager ") + QString(pCompileInfoVersion);
+            if (CONFIG (AvoidEncaseProblems))
+            {
+               GuymagerVersion = GuymagerVersion.left (THREADWRITE_ENCASE_MAXLEN_IMAGERVERSION);
+               Uname           = Uname          .left (THREADWRITE_ENCASE_MAXLEN_OSVERSION    );
+            }
+
             CHK_LIBEWF (libewf_set_header_value (pFile, (char *)"acquiry_operating_system", STR_AND_LEN(Uname)))
             CHK_LIBEWF (libewf_set_header_value (pFile, (char *)"acquiry_software_version", STR_AND_LEN(GuymagerVersion)))
 
@@ -413,14 +454,14 @@ class t_OutputFileEWF: public t_OutputFile
 
          if (Written != Size)
          {
-            LOG_ERROR ("Written %d/%d bytes", Written, Size)
+            LOG_ERROR ("[%s] Written %d/%d bytes", QSTR_TO_PSZ (poDevice->LinuxDevice), Written, Size)
             return ERROR_THREADWRITE_WRITE_FAILED;
          }
 
          return NO_ERROR;
       }
 
-      APIRET Verify (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, quint64 *pPos)
+      APIRET Verify (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, t_Device::t_ImageFileHashList &ImageFileHashList, quint64 *pPos)
       {
          quint64      Remaining;
          unsigned int ToRead;
@@ -432,13 +473,20 @@ class t_OutputFileEWF: public t_OutputFile
             poVerifyBuff = UTIL_MEM_ALLOC (poDevice->FifoBlockSize);
             if (poVerifyBuff == NULL)
                CHK (ERROR_THREADWRITE_MALLOC_FAILED);
+
+            for (int i=0; i<oImageFileCount; i++)
+               ImageFileHashList.append (new t_Device::t_ImageFileHash (poImageFilenameArr[i], NULL));  // No MD5 files hashes for EWF files generated by libewf
          }
          Remaining = poDevice->Size - *pPos;
          ToRead = GETMIN (Remaining, poDevice->FifoBlockSize);
+
+         CHK_EXIT (poThreadWrite->SetDebugMessage ("Verification: Calling libewf_read_buffer"))
          Read = libewf_read_buffer (poFile, poVerifyBuff, ToRead);
+         CHK_EXIT (poThreadWrite->SetDebugMessage ("Verification: Returning from libewf_read_buffer"))
+
          if (Read != (ssize_t)ToRead)
          {
-            LOG_INFO ("Reading from EWF file failed (%zu)", Read);
+            LOG_INFO ("[%s] Reading from EWF file failed (%zu)", QSTR_TO_PSZ (poDevice->LinuxDevice), Read);
             return ERROR_THREADWRITE_VERIFY_FAILED;
          }
          if (pHashContextMD5)    CHK_EXIT (HashMD5Append    (pHashContextMD5   , poVerifyBuff, ToRead))
@@ -451,19 +499,52 @@ class t_OutputFileEWF: public t_OutputFile
 
       APIRET Close (void)
       {
-         int rc;
-         int i;
+         QList<quint64> BadSectors;
+         quint64        i, Count;
+         quint64        From, To, Next;
+         int            rc;
+         int            j;
 
          if (poFile == NULL)
             CHK (ERROR_THREADWRITE_NOT_OPENED)
 
+         // Write bad sector list
+         // ---------------------
+         if (!oVerification)
+         {
+            CHK (poDevice->GetBadSectors (BadSectors, false))
+            Count = BadSectors.count();
+            if (Count)
+            {
+               i = 0;
+               while (i<Count)
+               {
+                  From = BadSectors.at(i++);
+                  To   = From;
+                  while (i < Count)
+                  {
+                     Next = BadSectors.at(i);
+                     if (Next != To+1)
+                         break;
+                     To = Next;
+                     i++;
+                  }
+                  CHK_LIBEWF (libewf_add_acquiry_error(poFile, From, To - From + 1))
+               }
+            }
+         }
+
+         // Write hash
+         // ----------
          if (!oVerification && poDevice->HasCompressionThreads() && poDevice->Acquisition.CalcMD5)
             CHK_LIBEWF (libewf_set_md5_hash (poFile, (uint8_t*)&poDevice->MD5Digest, HASH_MD5_DIGEST_LENGTH))
 
+         // Close and cleanup
+         // -----------------
          rc = libewf_close (poFile);
          if (rc != 0)
          {
-            LOG_ERROR ("Error in libewf function: libewf_close, rc=%d", rc)
+            LOG_ERROR ("[%s] Error in libewf function: libewf_close, rc=%d", QSTR_TO_PSZ (poDevice->LinuxDevice), rc)
             return ERROR_THREADWRITE_LIBEWF_FAILED;
          }
 
@@ -473,8 +554,8 @@ class t_OutputFileEWF: public t_OutputFile
             poVerifyBuff = NULL;
          }
 
-         for (i=0; i<oImageFileCount; i++)
-            free (poImageFilenameArr[i]);
+         for (j=0; j<oImageFileCount; j++)
+            free (poImageFilenameArr[j]);
          free (poImageFilenameArr);
 
          poFile = NULL;
@@ -501,11 +582,15 @@ class t_OutputFileEWF: public t_OutputFile
       bool            oHasCompressionThreads;
 };
 
+// ------------------------------------------------------------------------------------------------
+//                                           AAFF FORMAT
+// ------------------------------------------------------------------------------------------------
 
-class t_OutputFileAFF: public t_OutputFile
+class t_OutputFileAAFF: public t_OutputFile
 {
    public:
-      t_OutputFileAFF (void) :
+      t_OutputFileAAFF (t_ThreadWrite *pThreadWrite) :
+         t_OutputFile          (pThreadWrite),
          poFile                (NULL ),
          poDevice              (NULL ),
          poVerifyBuff          (NULL ),
@@ -514,10 +599,10 @@ class t_OutputFileAFF: public t_OutputFile
       {
       }
 
-      ~t_OutputFileAFF (void)
+      ~t_OutputFileAAFF (void)
       {
-         if (t_OutputFileAFF::Opened())
-            (void) t_OutputFileAFF::Close();
+         if (t_OutputFileAAFF::Opened())
+            (void) t_OutputFileAAFF::Close();
       } //lint !e1740
 
       APIRET Open (t_pDevice pDevice, bool Verify)
@@ -540,16 +625,21 @@ class t_OutputFileAFF: public t_OutputFile
 
          if (oVerification)
          {
-            rc = AaffOpen (&pFile, QSTR_TO_PSZ(FileName));
+            rc = AaffOpen (&pFile, QSTR_TO_PSZ(FileName), pDevice->Size);
             if (rc)
             {
-               LOG_INFO ("AFF image verification failed: %s", ToolErrorMessage (rc))
+               LOG_INFO ("[%s] AFF image verification failed: %s", QSTR_TO_PSZ (poDevice->LinuxDevice), ToolErrorMessage (rc))
                return ERROR_THREADWRITE_VERIFY_FAILED;
             }
          }
          else
          {
-            CHK (AaffOpen (&pFile, QSTR_TO_PSZ(FileName), pDevice->Size, pDevice->SectorSize, pDevice->FifoBlockSize))
+            rc = AaffOpen (&pFile, QSTR_TO_PSZ(FileName), pDevice->Size, pDevice->SectorSize, pDevice->FifoBlockSize);
+            if (rc)
+            {
+               LOG_INFO ("[%s] Opening AFF image for writing failed: %s", QSTR_TO_PSZ (poDevice->LinuxDevice), ToolErrorMessage (rc))
+               return ERROR_THREADWRITE_OPEN_FAILED;
+            }
 
             rc = ToolSysInfoGetMacAddr (&MacAddr);
             if (rc == TOOLSYSINFO_ERROR_NO_ADDR)
@@ -586,6 +676,7 @@ class t_OutputFileAFF: public t_OutputFile
       //lint -save -esym(613,t_OutputFileAFF::poDevice)   Possible use of null pointer
       APIRET Write (t_pFifoBlock pFifoBlock)
       {
+         APIRET rc;
          if (!oHasCompressionThreads) // Do preprocessing if not already done in compression threads
          {
             t_pFifoBlock pPreprocessBlock;
@@ -605,16 +696,21 @@ class t_OutputFileAFF: public t_OutputFile
             }
          }
 
-         CHK (AaffWrite (poFile, pFifoBlock->pAaffPreprocess, pFifoBlock->Buffer, pFifoBlock->DataSize))
+         rc = AaffWrite (poFile, pFifoBlock->pAaffPreprocess, pFifoBlock->Buffer, pFifoBlock->DataSize);
+         if (rc != NO_ERROR)
+            CHK (ERROR_THREADWRITE_WRITE_FAILED)
 
          return NO_ERROR;
       }
       //lint -restore
 
-      APIRET Verify (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, quint64 *pPos)
+      APIRET Verify (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, t_Device::t_ImageFileHashList &ImageFileHashList, quint64 *pPos)
       {
-         unsigned int DataLen = poDevice->FifoBlockSize;
-         APIRET       rc;
+         unsigned int    DataLen = poDevice->FifoBlockSize;
+         APIRET          rc;
+         QString         ImageFilename;
+         t_HashMD5Digest MD5Digest;
+         bool            MD5Valid;
 
          if (*pPos == 0)
          {
@@ -622,10 +718,10 @@ class t_OutputFileAFF: public t_OutputFile
             if (poVerifyBuff == NULL)
                CHK (ERROR_THREADWRITE_MALLOC_FAILED);
          }
-         rc = AaffReadNextPage (poFile, (unsigned char *) poVerifyBuff, &DataLen);
+         rc = AaffReadNextPage (poFile, (unsigned char *) poVerifyBuff, &DataLen, &ImageFilename, &MD5Digest, &MD5Valid);
          if (rc)
          {
-            LOG_INFO ("AFF image verification failed: %s", ToolErrorMessage (rc))
+            LOG_INFO ("[%s] AFF image verification failed: %s", QSTR_TO_PSZ (poDevice->LinuxDevice), ToolErrorMessage (rc))
             return ERROR_THREADWRITE_VERIFY_FAILED;
          }
 
@@ -634,6 +730,14 @@ class t_OutputFileAFF: public t_OutputFile
 
          *pPos += DataLen;
 
+         if (!ImageFilename.isEmpty())
+         {
+            QString MD5Str;
+
+            CHK (HashMD5DigestStr (&MD5Digest, MD5Str))
+            ImageFileHashList.append (new t_Device::t_ImageFileHash (ImageFilename, &MD5Digest, MD5Valid));
+         }
+
          return NO_ERROR;
       }
 
@@ -687,6 +791,194 @@ class t_OutputFileAFF: public t_OutputFile
       bool        oVerification;
 };
 
+// ------------------------------------------------------------------------------------------------
+//                                           AEWF FORMAT
+// ------------------------------------------------------------------------------------------------
+
+class t_OutputFileAEWF: public t_OutputFile
+{
+   public:
+      t_OutputFileAEWF (t_ThreadWrite *pThreadWrite) :
+         t_OutputFile           (pThreadWrite),
+         poFile                 (NULL ),
+         poDevice               (NULL ),
+         poVerifyBuff           (NULL ),
+          oHasCompressionThreads(false),
+          oVerification         (false)
+      {
+      }
+
+      ~t_OutputFileAEWF (void)
+      {
+         if (t_OutputFileAEWF::Opened())
+            (void) t_OutputFileAEWF::Close();
+      } //lint !e1740
+
+      APIRET Open (t_pDevice pDevice, bool Verify)
+      {
+         t_pAewf pFile = NULL;
+         QString  FileName;
+         QString  GuymagerVersion;
+         QString  Uname;
+
+         poDevice      = pDevice;
+         oVerification =  Verify;
+         oHasCompressionThreads = pDevice->HasCompressionThreads();
+
+         FileName = pDevice->Acquisition.ImagePath + pDevice->Acquisition.ImageFilename;
+         if (oVerification)
+         {
+            CHK (AewfOpen (&pFile, QSTR_TO_PSZ(FileName)))
+         }
+         else
+         {
+            CHK (ToolSysInfoUname (Uname))
+            GuymagerVersion = QString("guymager ") + QString(pCompileInfoVersion);
+            if (CONFIG (AvoidEncaseProblems))
+            {
+               GuymagerVersion = GuymagerVersion.left (THREADWRITE_ENCASE_MAXLEN_IMAGERVERSION);
+               Uname           = Uname          .left (THREADWRITE_ENCASE_MAXLEN_OSVERSION    );
+            }
+            CHK (AewfOpen (&pFile, QSTR_TO_PSZ(FileName), pDevice->Size, pDevice->Acquisition.SplitFileSize, pDevice->FifoBlockSize, pDevice->SectorSize,
+                                   pDevice->Removable ? AEWF_MEDIATYPE_REMOVABLE : AEWF_MEDIATYPE_FIXED,
+                                   AEWF_MEDIAFLAGS_PHYSICAL,
+                                   pDevice->Acquisition.Description, pDevice->Acquisition.CaseNumber, pDevice->Acquisition.EvidenceNumber,
+                                   pDevice->Acquisition.Examiner   , pDevice->Acquisition.Notes     , pDevice->Model                     ,
+                                   pDevice->SerialNumber           , GuymagerVersion                , Uname))
+         }
+         poFile = pFile; // Only set poFile at the very end, so the CompressionThreads won't use it until everything is initialised
+         return NO_ERROR;
+      }
+
+      APIRET Write (t_pFifoBlock pFifoBlock)
+      {
+         APIRET rc;
+
+         if (!oHasCompressionThreads) // Do preprocessing if not already done in compression threads
+         {
+            t_pFifoBlock pPreprocessBlock;
+
+            CHK_EXIT (t_Fifo::Create (poDevice->pFifoMemory, pPreprocessBlock, poDevice->FifoAllocBlockSize))
+            pPreprocessBlock->DataSize = pFifoBlock->DataSize;
+            CHK_EXIT (AewfPreprocess (poFile, &pPreprocessBlock->pAewfPreprocess, pFifoBlock      ->Buffer, pFifoBlock      ->DataSize,
+                                                                                  pPreprocessBlock->Buffer, pPreprocessBlock->BufferSize))
+            if (pPreprocessBlock->pAewfPreprocess->Compressed)
+            {
+               CHK_EXIT (t_Fifo::Destroy (poDevice->pFifoMemory, pFifoBlock))
+               pFifoBlock = pPreprocessBlock;
+            }
+            else
+            {
+               pFifoBlock->pAewfPreprocess = pPreprocessBlock->pAewfPreprocess;
+               CHK_EXIT (t_Fifo::Destroy (poDevice->pFifoMemory, pPreprocessBlock))
+            }
+         }
+         rc = AewfWrite (poFile, pFifoBlock->pAewfPreprocess, pFifoBlock->Buffer, pFifoBlock->pAewfPreprocess->DataLenOut);
+         if (rc != NO_ERROR)
+            CHK (ERROR_THREADWRITE_WRITE_FAILED)
+
+         return NO_ERROR;
+      }
+      //lint -restore
+
+   private:
+      APIRET ProcessUnit (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, t_Device::t_ImageFileHashList &ImageFileHashList, quint64 *pPos, bool *pFinished)
+      {
+         QString         SegmentFilename;
+         QString         MD5Str;
+         t_HashMD5Digest MD5Digest;
+         bool            MD5Valid;
+         unsigned int    DataLen;
+         APIRET          rc;
+
+         DataLen = poDevice->FifoAllocBlockSize;
+         rc = AewfReadNextChunk (poFile, (unsigned char *) poVerifyBuff, &DataLen, &SegmentFilename, &MD5Digest, &MD5Valid, pFinished);
+         if (rc)
+         {
+            LOG_INFO ("[%s] AEF image verification failed: %s", QSTR_TO_PSZ (poDevice->LinuxDevice), ToolErrorMessage (rc))
+            return ERROR_THREADWRITE_VERIFY_FAILED;
+         }
+
+         if (DataLen)
+         {
+            if (pHashContextMD5)    CHK_EXIT (HashMD5Append    (pHashContextMD5   , poVerifyBuff, DataLen))
+            if (pHashContextSHA256) CHK_EXIT (HashSHA256Append (pHashContextSHA256, poVerifyBuff, DataLen))
+            *pPos += DataLen;
+         }
+
+         if (!SegmentFilename.isEmpty())
+         {
+            CHK (HashMD5DigestStr (&MD5Digest, MD5Str))
+            ImageFileHashList.append (new t_Device::t_ImageFileHash (SegmentFilename, &MD5Digest, MD5Valid));
+         }
+         return NO_ERROR;
+      }
+
+   public:
+      APIRET Verify (t_pHashContextMD5 pHashContextMD5, t_pHashContextSHA256 pHashContextSHA256, t_Device::t_ImageFileHashList &ImageFileHashList, quint64 *pPos)
+      {
+         bool Finished;
+
+         if (*pPos == 0)
+         {
+            poVerifyBuff = UTIL_MEM_ALLOC (poDevice->FifoAllocBlockSize);
+            if (poVerifyBuff == NULL)
+               CHK (ERROR_THREADWRITE_MALLOC_FAILED);
+         }
+         CHK (ProcessUnit (pHashContextMD5, pHashContextSHA256, ImageFileHashList, pPos, &Finished))
+
+         if (*pPos >= poDevice->Size)  // If all data has been read keep calling
+         {                             // the processing function until it says it
+            while (!Finished)          // finished (see AewfReadNextChunk)
+               CHK (ProcessUnit (pHashContextMD5, pHashContextSHA256, ImageFileHashList, pPos, &Finished))
+         }
+         return NO_ERROR;
+      }
+
+      APIRET Close (void)
+      {
+         QList<quint64> BadSectors;
+
+         if (poFile == NULL)
+            CHK (ERROR_THREADWRITE_NOT_OPENED)
+
+         if (oVerification)
+         {
+            CHK (AewfClose (&poFile))
+         }
+         else
+         {
+            CHK (poDevice->GetBadSectors (BadSectors, false))
+            CHK (AewfClose (&poFile, BadSectors, (unsigned char*)&poDevice->MD5Digest, (unsigned char*)&poDevice->SHA256Digest))
+         }
+         poFile = NULL;
+
+         return NO_ERROR;
+      }
+      //lint -restore
+
+      void *GetFileHandle (void)
+      {
+         return poFile;
+      }
+
+      bool Opened (void)
+      {
+         return (poFile != NULL);
+      }
+
+   private:
+      t_pAewf    poFile;
+      t_pDevice  poDevice;
+      void      *poVerifyBuff;
+      bool        oHasCompressionThreads;
+      bool        oVerification;
+};
+
+
+// ------------------------------------------------------------------------------------------------
+//                                           WRITE THREAD
+// ------------------------------------------------------------------------------------------------
 
 class t_ThreadWriteLocal
 {
@@ -823,7 +1115,8 @@ void t_ThreadWrite::run (void)
    quint64        Written   = 0;
    APIRET         rc;
 
-   LOG_INFO ("Acquisition of %s: Write thread started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
+   LOG_INFO ("[%s] Write thread started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
+   CHK_EXIT (SetDebugMessage ("Start run function"))
 
    pDevice = pOwn->pDevice;
    pDevice->SetCurrentWritePos (0LL);
@@ -832,16 +1125,18 @@ void t_ThreadWrite::run (void)
 
    switch (pDevice->Acquisition.Format)
    {
-      case t_File::DD:  pOutputFile = new t_OutputFileDD;  break;
-      case t_File::EWF: pOutputFile = new t_OutputFileEWF; break;
-      case t_File::AFF: pOutputFile = new t_OutputFileAFF; break;
+      case t_File::EWF : if (CONFIG(EwfFormat) == t_File::AEWF)
+                              pOutputFile = new t_OutputFileAEWF(this);
+                         else pOutputFile = new t_OutputFileEWF (this); break;
+      case t_File::DD  :      pOutputFile = new t_OutputFileDD  (this); break;
+      case t_File::AAFF:      pOutputFile = new t_OutputFileAAFF(this); break;
       default: CHK_EXIT (ERROR_THREADWRITE_INVALID_FORMAT)
    }
 
    rc = pOutputFile->Open (pDevice, false);
    if (rc == ERROR_THREADWRITE_OPEN_FAILED)
    {
-      LOG_INFO ("Could not open destination file %s", QSTR_TO_PSZ (pOwn->pDevice->Acquisition.ImageFilename))
+      LOG_INFO ("[%s] Could not open destination file %s", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), QSTR_TO_PSZ (pOwn->pDevice->Acquisition.ImageFilename))
       pDevice->AbortReason  = t_Device::ThreadWriteWriteError;
       pDevice->AbortRequest = true;
    }
@@ -859,13 +1154,15 @@ void t_ThreadWrite::run (void)
       if (*(pOwn->pSlowDownRequest))
          msleep (THREADWRITE_SLOWDOWN_SLEEP);
 
+      CHK_EXIT (SetDebugMessage ("Wait for block"))
       CHK_EXIT (pDevice->pFifoWrite->Get (pFifoBlock))
+      CHK_EXIT (SetDebugMessage ("Process block"))
       if (pFifoBlock)
       {
 //         t_Fifo::LogBlock (pDevice->pFifoMemory, pFifoBlock, 'w');
          if (pFifoBlock->Nr != Blocks)
          {
-            LOG_ERROR ("Fifo block number out of sequence. Expected: %Ld Received: %Ld", Blocks, pFifoBlock->Nr)
+            LOG_ERROR ("[%s] Fifo block number out of sequence. Expected: %Ld Received: %Ld", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), Blocks, pFifoBlock->Nr)
 //            t_Fifo::LogBlock (pDevice->pFifoMemory, pFifoBlock, 'e');
             CHK_EXIT (ERROR_THREADWRITE_OUT_OF_SEQUENCE_BLOCK)
         }
@@ -877,8 +1174,8 @@ void t_ThreadWrite::run (void)
              (rc == ERROR_THREADWRITE_OPEN_FAILED))
          {
             const char *pAction = (rc == ERROR_THREADWRITE_WRITE_FAILED) ? "write to" : "open";
-            LOG_ERROR ("Could not %s destination file %s", pAction, QSTR_TO_PSZ (pOwn->pDevice->Acquisition.ImageFilename))
-            LOG_INFO ("Last block sizes: %d - %d - %zd ", pFifoBlock->BufferSize, pFifoBlock->DataSize, pFifoBlock->EwfDataSize)
+            LOG_ERROR ("[%s] Could not %s destination file %s", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), pAction, QSTR_TO_PSZ (pOwn->pDevice->Acquisition.ImageFilename))
+            LOG_INFO  ("[%s] Last block sizes: %d - %d - %zd ", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), pFifoBlock->BufferSize, pFifoBlock->DataSize, pFifoBlock->EwfDataSize)
             pDevice->AbortReason  = t_Device::ThreadWriteWriteError;
             pDevice->AbortRequest = true;
          }
@@ -896,14 +1193,14 @@ void t_ThreadWrite::run (void)
       }
       else
       {
-         LOG_INFO ("Dummy block")
+         LOG_INFO ("[%s] Dummy block", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
          Finished = true;
       }
    }
 
    // Wait until file handle released and close it
    // --------------------------------------------
-   LOG_INFO ("Waiting for all other threads using the file handle to finish")
+   LOG_INFO ("[%s] Waiting for all other threads using the file handle to finish", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
    do
    {
       bool AskedForRelease = false;
@@ -922,7 +1219,7 @@ void t_ThreadWrite::run (void)
       }
    } while (!FileHandleReleased);
 
-   LOG_INFO ("Closing output file")
+   LOG_INFO ("[%s] Closing output file", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
    if (pOwn->pOutputFile->Opened())
    {
       rc = pOwn->pOutputFile->Close ();
@@ -951,12 +1248,13 @@ void t_ThreadWrite::run (void)
            CHK_EXIT (HashSHA256Init (pHashContextSHA256))
       else pHashContextSHA256 = NULL;
 
-      LOG_INFO ("Reopening output file for verification")
-
+      LOG_INFO ("[%s] Reopening image file for verification", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
+      CHK_EXIT (SetDebugMessage ("pOutputFile->Open verification"))      
       rc = pOutputFile->Open (pDevice, true);
+      CHK_EXIT (SetDebugMessage ("Returning from pOutputFile->Open"))
       if (rc == ERROR_THREADWRITE_OPEN_FAILED)
       {
-         LOG_INFO ("Could not open destination file %s for verification", QSTR_TO_PSZ (pOwn->pDevice->Acquisition.ImageFilename))
+         LOG_INFO ("[%s] Could not open image file %s for verification", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), QSTR_TO_PSZ (pOwn->pDevice->Acquisition.ImageFilename))
          pDevice->AbortReason  = t_Device::ThreadWriteVerifyError;
          pDevice->AbortRequest = true;
       }
@@ -968,10 +1266,10 @@ void t_ThreadWrite::run (void)
       Finished = false;
       while (!Finished && !pDevice->AbortRequest)
       {
-         rc = pOwn->pOutputFile->Verify (pHashContextMD5, pHashContextSHA256, &Pos);
+         rc = pOwn->pOutputFile->Verify (pHashContextMD5, pHashContextSHA256, pDevice->ImageFileHashList, &Pos);
          if (rc == ERROR_THREADWRITE_VERIFY_FAILED)
          {
-            LOG_ERROR ("Could not verify image %s", QSTR_TO_PSZ (pOwn->pDevice->Acquisition.ImageFilename))
+            LOG_ERROR ("[%s] Could not verify image %s", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), QSTR_TO_PSZ (pOwn->pDevice->Acquisition.ImageFilename))
             pDevice->AbortReason  = t_Device::ThreadWriteVerifyError;
             pDevice->AbortRequest = true;
          }
@@ -987,7 +1285,7 @@ void t_ThreadWrite::run (void)
             }
          }
       }
-      LOG_INFO ("Verification finished, closing output file")
+      LOG_INFO ("[%s] Verification finished, closing output file", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
       if (pOwn->pOutputFile->Opened())
       {
          rc = pOwn->pOutputFile->Close ();
@@ -1010,7 +1308,8 @@ void t_ThreadWrite::run (void)
       CHK_EXIT (DeleteImageFiles (true))
    }
 
-   LOG_INFO ("Write thread exits now (device %s, %Ld blocks processed, %Ld bytes written to output file)", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), Blocks, pDevice->GetCurrentWritePos ())
+   LOG_INFO ("[%s] Write thread exits now (%Ld blocks processed, %Ld bytes written to output file)", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), Blocks, pDevice->GetCurrentWritePos ())
+   CHK_EXIT (SetDebugMessage ("Exit run function"))
 }
 //lint -restore
 
@@ -1032,7 +1331,7 @@ static APIRET DeleteImageFiles0 (t_pDevice pDevice, const QDir &Dir, const QStri
       if (Success)
            Info += "successfull";
       else Info += "could not be deleted";
-      LOG_INFO ("%s", QSTR_TO_PSZ(Info))
+      LOG_INFO ("[%s] %s", QSTR_TO_PSZ (pDevice->LinuxDevice), QSTR_TO_PSZ(Info))
    }
 
    return NO_ERROR;
@@ -1047,13 +1346,13 @@ APIRET t_ThreadWrite::DeleteImageFiles (bool AlsoDeleteInfoFile)
 
    if (!pDevice->Acquisition.Clone)
    {
-      LOG_INFO ("Deleting existing image files of the same name")
+      LOG_INFO ("[%s] Deleting existing image files of the same name", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
       CHK (t_File::GetFormatExtension (pDevice, &ExtensionImage))
       CHK (DeleteImageFiles0 (pDevice, DirImage, QStringList(pDevice->Acquisition.ImageFilename + ExtensionImage)))
    }
    if (AlsoDeleteInfoFile)
    {
-      LOG_INFO ("Deleting existing info file of the same name")
+      LOG_INFO ("[%s] Deleting existing info file of the same name", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
       CHK (DeleteImageFiles0 (pDevice, DirImage, QStringList(pDevice->Acquisition.InfoFilename + t_File::pExtensionInfo)))
    }
    CHK (pDevice->SetMessage (QString()))
